@@ -575,11 +575,20 @@ remove_events (const char *bus_name, const char *event)
           spi_event_is_subtype (evdata->data, remove_data))
         {
           GList *events = spi_global_app_data->events;
-          list = list->next;
           g_strfreev (evdata->data);
           g_free (evdata->bus_name);
           g_free (evdata);
-          spi_global_app_data->events = g_list_remove (events, evdata);
+          if (list->prev)
+            {
+              GList *next = list->next;
+              list->prev = g_list_remove (list->prev, evdata);
+              list = next;
+            }
+          else
+            {
+              spi_global_app_data->events = g_list_remove (events, evdata);
+              list = spi_global_app_data->events;
+            }
         }
       else
         {
@@ -786,7 +795,10 @@ atk_bridge_adaptor_init (gint * argc, gchar ** argv[])
   g_option_context_add_main_entries (opt, atspi_option_entries, NULL);
   g_option_context_set_ignore_unknown_options (opt, TRUE);
   if (!g_option_context_parse (opt, argc, argv, &err))
-    g_warning ("AT-SPI Option parsing failed: %s\n", err->message);
+    {
+      g_warning ("AT-SPI Option parsing failed: %s\n", err->message);
+      g_error_free (err);
+    }
   g_option_context_free (opt);
 
   /* Allocate global data and do ATK initializations */
@@ -996,6 +1008,7 @@ spi_atk_remove_client (const char *bus_name)
       clients = g_slist_delete_link (clients, l);
       if (!clients)
         spi_atk_deregister_event_listeners ();
+      return;
     }
 
     l = next_node;
